﻿using System;
using System.Collections.Generic;
using System.Text;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Problems.DataAnalysis.Transformations {
  [Item("LogarithmicTransformation", "Represents a logarithmic transformation.")]
  public class LogarithmicTransformation : Transformation<double> {
    protected const string BaseParameterName = "Base";

    #region Parameters
    public IValueParameter<DoubleValue> BaseParameter {
      get { return (IValueParameter<DoubleValue>)Parameters[BaseParameterName]; }
    }
    #endregion

    #region properties
    public double Base {
      get { return BaseParameter.Value.Value; }
    }
    #endregion

    [StorableConstructor]
    protected LogarithmicTransformation(bool deserializing) : base(deserializing) { }
    protected LogarithmicTransformation(Transformation<double> original, Cloner cloner)
      : base(original, cloner) {
    }
    public LogarithmicTransformation(IEnumerable<string> allowedColumns)
      : base(allowedColumns) {
      Parameters.Add(new ValueParameter<DoubleValue>(BaseParameterName, "Base of log-function", new DoubleValue(Math.E)));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new LogarithmicTransformation(this, cloner);
    }

    public override IEnumerable<double> Apply(IEnumerable<double> data) {
      foreach (double i in data) {
        if (i > 0.0)
          yield return Math.Log(i, Base);
        else
          yield return i;
      }
    }

    public override bool Check(IEnumerable<double> data, out string errorMsg) {
      StringBuilder sb = new StringBuilder();
      errorMsg = null;
      int errorCounter = 0;

      foreach (double i in data) {
        if (i <= 0.0) {
          ++errorCounter;
        }
      }

      if (errorCounter > 0) {
        sb.Append(String.Format("{0} values are zero or below zero. Logarithm can not be applied onto these values", errorCounter));
        errorMsg = sb.ToString();
        return false;
      }
      return true;
    }

  }
}
